import json
import time
from typing import NoReturn

from httpx import AsyncClient
from websockets.asyncio.client import connect

from odyssey_exchange_api.requests.base import BaseRequest, SignedRequest, ResponseType, WebsocketRequest
from .client import OdysseyExchangeAPI
from ..base import BASE_WEBSOCKET_URL
from ..responses import WebsocketResponse


class AsyncOdysseyExchangeAPI(OdysseyExchangeAPI):
    def __init__(
            self, api_key: str,
            secret_key: str,
            client_parameters: dict = None
    ):
        """
        Class for asynchronous work with API.

        :param api_key: your API key
        :param secret_key: your secret key
        :param client_parameters: optional dict with parameters to httpx.Client
        """

        super().__init__(api_key, secret_key)

        if client_parameters is None:
            client_parameters = {}

        self.__api_key = api_key
        self.__secret_key = secret_key
        self.__client = AsyncClient(**client_parameters)

    async def init_websocket_connection(self) -> NoReturn:
        self.__websocket_client = await connect(BASE_WEBSOCKET_URL)

    async def make_request(self, api_request: BaseRequest[ResponseType]) -> ResponseType:
        """
        Makes a asynchronous call to the api

        :param api_request: any of API requests which inherits from :class:`BaseRequest`.
        :return: an object specified at API request
        """

        if isinstance(api_request, SignedRequest):
            timestamp = str(int(time.time()) * 1000)
            api_request.sign(
                timestamp=timestamp,
                api_key=self.__api_key,
                secret_key=self.__secret_key
            )

        request = api_request.build_request()
        response_obj = await self.__client.send(request=request)
        response = self.process_response(response_obj)
        return api_request.make_response(response)

    async def make_websocket_request(self, api_request: WebsocketRequest):
        """
        Makes asynchronous call to the websocket connection

        :param api_request: any of API requests which inherits from :class:`WebsocketRequest`.
        :return:
        """
        data = api_request.build_request_data()
        if not isinstance(data, str) and not isinstance(data, bytes):
            data = json.dumps(data)

        await self.__websocket_client.send(data)

    async def receive_websocket_message(self) -> WebsocketResponse:
        """
        Get a message from a websocket

        :return:
        """
        message = await self.__websocket_client.recv()
        response = self.process_websocket_message(message)
        return response



from .futures import FuturesContract, FuturesServerTime, FuturesTickerData, FuturesIndexTagPrice, FuturesKlineData, \
    FuturesOrder, FuturesHistoricalCommission, FuturesProfitAndLoss, FuturesMyTrade, FuturesTriggerOrder, \
    FuturesAccountPosition, FuturesAccountPositions, FuturesAccountCoinInfo
from .spot import SpotOrder, SpotServerTime, SpotTickerData, SpotKlineData, SpotRecentTrade, SpotAssetBalance, \
    SpotAssetPair, SpotMyTrade, SpotSingleBatchOrder
from .websocket import WebsocketHistoryTrade, WebsocketFullDepth, WebsocketKlineData, WebsocketTickerData



from decimal import Decimal

from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.enums import SpotOrderSide, SpotOrderType
from odyssey_exchange_api.objects import SpotOrder
from odyssey_exchange_api.requests.base import SignedRequest


class SpotCreateOrderRequest(SignedRequest[SpotOrder]):
    """
    Create a new order. Returns a :class:`odyssey_exchange_api.objects.SpotOrder`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#create-new-order
    """

    _request_url = BASE_SPOT_URL
    _request_method = "POST"
    _request_path = "/sapi/v1/order"

    __returning__ = SpotOrder

    symbol: str
    """The uppercase symbol name, e.g., BTCUSDT."""
    volume: Decimal
    """Order quantity, there is a precision limit."""
    side: SpotOrderSide
    """Order side."""
    type: SpotOrderType
    """Order type."""

    price: Decimal | None = None
    """Order price, for LIMIT orders must be sent, there is a precision limit."""
    newClientOrderId: str | None = None
    """Client order ID."""



from .account_position import FuturesAccountPosition
from .account_positions import FuturesAccountPositions
from .account_coin_info import FuturesAccountCoinInfo
from .contract import FuturesContract
from .historical_commission import FuturesHistoricalCommission
from .index_tag_price import FuturesIndexTagPrice
from .kline_data import FuturesKlineData
from .my_trade import FuturesMyTrade
from .order import FuturesOrder
from .profit_and_loss import FuturesProfitAndLoss
from .server_time import FuturesServerTime
from .ticker_data import FuturesTickerData
from .trigger_order import FuturesTriggerOrder



from decimal import Decimal

from pydantic import BaseModel


class FuturesTickerData(BaseModel):
    """
    An object with information about a contract per day.
    """

    time: int
    """Timestamp"""
    high: Decimal
    """Highest price"""
    low: Decimal
    """Lowest price"""
    last: Decimal
    """Latest deal price"""
    vol: Decimal
    """Trade volume"""
    buy: Decimal
    """The price in the buying book order at the first one"""
    sell: Decimal
    """The price in the selling book order at the first one"""
    rose: str
    """Range of increase and decrease, + is increase, - is decrease, +0.05 means increase by 5%"""



from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.requests.base import BaseRequest
from odyssey_exchange_api.responses import SpotDepthResponse


class SpotDepthRequest(BaseRequest[SpotDepthResponse]):
    """
    Market depth data. Returns a :class:`odyssey_exchange_api.responses.SpotDepthResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#depth
    """

    _request_url = BASE_SPOT_URL
    _request_method = "GET"
    _request_path = "/sapi/v1/depth"

    __returning__ = SpotDepthResponse

    symbol: str
    """The uppercase symbol name, e.g., BTCUSDT."""
    limit: int = 100
    """The maximum number of items that can be returned in the query result. The value must be a positive integer. The default value is 100, maximum value is 100."""



from decimal import Decimal

from pydantic import BaseModel


class SpotAssetBalance(BaseModel):
    """
    An object that contains information about the amount of assets, available assets, and locked assets.
    """

    asset: str
    """Coin name"""
    free: Decimal
    """Account available amount"""
    locked: Decimal
    """Account frozen amount"""



from decimal import Decimal

from pydantic import BaseModel


class FuturesDepthResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.FuturesDepthRequest`.

    Contains an arrays with price and volume.
    """

    asks: list[list[Decimal]]
    """Order book selling information, the array length is 2, subscript one is the price, type is float; subscript two is the quantity corresponding to the current price, type is float"""
    bids: list[list[Decimal]]
    """Order book buying information, the array length is 2, subscript one is the price, type is float; subscript two is the quantity corresponding to the current price, type is float"""
    time: int | None
    """Current timestamp, nullable"""



import json
import zlib
from typing import Any, NoReturn

from httpx import Response

from odyssey_exchange_api.exceptions import UnknownException, EXCEPTIONS_DICT
from odyssey_exchange_api.requests.base import BaseRequest, ResponseType
from odyssey_exchange_api.responses import WebsocketResponse
from odyssey_exchange_api.utils import resolve_websocket_response_class


class OdysseyExchangeAPI:
    def __init__(
            self,
            api_key: str,
            secret_key: str,
    ):
        """
        The base class of the API client, use it to inherit your own clients.

        :param api_key: your API key
        :param secret_key: your secret key
        """

        self.__api_key = api_key
        self.__secret_key = secret_key
        self.__websocket_client = None

    def process_response(self, response: Response) -> Any:
        """
        Checks the response from the api, if there is an error raises the error, if not returns data from the response

        :param response: :class:`httpx.Response`
        :return: any
        """
        try:
            response_data = response.json()
        except:
            raise UnknownException(response.content)

        if not isinstance(response_data, dict):
            return response_data

        code = response_data.get("code")
        if code and code != '0':
            exc_class = EXCEPTIONS_DICT.get(int(code))
            if not exc_class:
                raise UnknownException(response_data)

            data = response_data.get("data")
            raise exc_class(data)
        else:
            if 200 < response.status_code < 300:
                raise UnknownException(response_data)
            return response_data

    def process_websocket_message(self, message: bytes) -> WebsocketResponse:
        decoded_message = zlib.decompress(message, 16 + zlib.MAX_WBITS)
        data = json.loads(decoded_message)
        response_obj = resolve_websocket_response_class(data)
        return response_obj(**data)

    def init_websocket_connection(self) -> NoReturn:
        """
        Initialize the websocket client for further work with the websocket API.
        """
        raise NotImplementedError

    def make_request(self, api_request: BaseRequest[ResponseType]) -> ResponseType:
        """
        Makes a call to the api

        :param api_request: any of API requests which inherits from :class:`BaseRequest`.
        :return: an object specified at API request
        """
        raise NotImplementedError



from pydantic import BaseModel


class SpotBatchOrdersResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.SpotBatchOrdersRequest`.

    Contains an array of created order_ids.
    """

    ids: list[str]
    """Array of created order ids"""



from pydantic import Field

from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.objects import SpotOrder
from odyssey_exchange_api.requests.base import SignedRequest


class SpotQueryOrderRequest(SignedRequest[SpotOrder]):
    """
    Get the order data. Returns a :class:`odyssey_exchange_api.objects.SpotOrder`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#query-order
    """
    _request_url = BASE_SPOT_URL
    _request_method = "GET"
    _request_path = "/sapi/v1/order"

    __returning__ = SpotOrder

    order_id: str = Field(serialization_alias="orderId")
    """Order ID."""
    symbol: str
    """The lowercase symbol name, e.g., btcusdt."""



from odyssey_exchange_api.enums import WebsocketEventType
from odyssey_exchange_api.requests.base import WebsocketRequest


class WebsocketFullDepthRequest(WebsocketRequest):
    event: WebsocketEventType
    symbol: str

    def build_request_data(self) -> dict:
        data = {
            "event": self.event,
            "params": {
                "channel": f"market_{self.symbol}_depth_step0",
            }
        }
        return data



import datetime
from decimal import Decimal

from pydantic import BaseModel, Field

from odyssey_exchange_api.enums import FuturesPositionType, FuturesOrderSide, FuturesPositionStatus, \
    FuturesPositionFrozenStatus


class FuturesAccountPosition(BaseModel):
    """
    An object that contains information about the user position.
    """

    id: int
    """Position id"""
    uid: int
    """User ID"""
    contract_id: int = Field(alias="contractId")
    """Contract ID"""
    position_type: FuturesPositionType = Field(alias="positionType")
    """Position type"""
    side: FuturesOrderSide
    """Position side"""
    volume: Decimal
    """Position quantity"""
    open_price: Decimal = Field(alias="openPrice")
    """Opening price"""
    average_price: Decimal = Field(alias="avgPrice")
    """Average position price"""
    close_price: Decimal = Field(alias="closePrice")
    """Average closing price"""
    leverage_level: int = Field(alias="leverageLevel")
    """Leverage multiple"""
    open_amount: Decimal = Field(alias="openAmount")
    """Opening margin"""
    hold_amount: Decimal = Field(alias="holdAmount")
    """Position margin"""
    close_volume: Decimal = Field(alias="closeVolume")
    """Quantity of positions closed"""
    pending_close_volume: Decimal = Field(alias="pendingCloseVolume")
    """The volume of place closing orders"""
    realized_amount: Decimal = Field(alias="realizedAmount")
    """Realized profit and loss"""
    history_realized_amount: Decimal = Field(alias="historyRealizedAmount")
    """Historical accumulated profit and loss"""
    trade_fee: Decimal = Field(alias="tradeFee")
    """Transaction Fees"""
    capital_fee: Decimal = Field(alias="capitalFee")
    """Funding charges"""
    close_profit: Decimal = Field(alias="closeProfit")
    """Position closing profit and loss"""
    share_amount: Decimal = Field(alias="shareAmount")
    """Sharing amount"""
    freeze_lock: FuturesPositionFrozenStatus = Field(alias="freezeLock")
    """Position frozen status"""
    status: FuturesPositionStatus
    """Position validity"""
    ctime: datetime.datetime
    """Creation time"""
    mtime: datetime.datetime
    """Update time"""
    brokerId: int
    """Broker id"""
    lock_time: datetime.datetime = Field(alias="lockTime")
    """Liquidation lock time"""
    margin_rate: Decimal = Field(alias="marginRate")
    """Margin rate"""
    reduce_price: Decimal = Field(alias="reducePrice")
    """Forced price reduction"""
    return_rate: Decimal = Field(alias="returnRate")
    """Rate of return (yield)"""
    unrealized_amount: Decimal = Field(alias="unRealizedAmount")
    """Unrealized profit or loss"""
    open_realized_amount: Decimal = Field(alias="openRealizedAmount")
    """Unrealized profit and loss on opening a position"""
    position_balance: Decimal = Field(alias="positionBalance")
    """Position value"""
    settle_profit: Decimal = Field(alias="settleProfit")
    """Position settlement"""
    index_price: Decimal = Field(alias="indexPrice")
    """Latest index price"""
    keep_rate: Decimal = Field(alias="keepRate")
    """Tiered Minimum Maintenance Margin Rate"""
    max_fee_rate: Decimal = Field(alias="maxFeeRate")
    """Maximum handling fee for closing the position"""



from odyssey_exchange_api.exceptions.base import BaseExchangeException, UnknownException
from odyssey_exchange_api.exceptions.rate_limit import RateLimitException
from odyssey_exchange_api.exceptions.general import DisconnectedException, UnknownExchangeException, \
    UnknownOrderCompositionException, \
    TooManyOrdersException, TooManyRequestsException, TimeoutAPIException, MissingAPIKeyException, \
    MissingSignatureException, MissingTimestampException, InvalidSignatureException, InvalidTimestampException, \
    UnexpectedResponseException, UnsupportedOperationException, NoThisCompanyException, \
    NoContentTypeException, ServiceShuttingDownException
from odyssey_exchange_api.exceptions.other import RestrictedTransactionsException, FrozenTransactionsException, \
    IllegalIPException, \
    WrongParameterException, InsufficientBalanceException, OrderNotExistException, IncorrectAPIOrIPException
from odyssey_exchange_api.exceptions.request import InvalidContractException, InvalidSideException, \
    InvalidSpotOrderTypeException, \
    NoPendingOrdersException, OrderPriceRangeException, NotCancellableOrderException, NotRequiredParamException, \
    UnknownParamException, RequiredParamException, RequiredParamEmptyException, UnreadParamException, \
    OrderMaximumLimitException, OrderVolumeNotEnoughException, BadPrecisionException, IllegalCharsException, \
    TooManyParametersException, MarketTradingDisabledException

EXCEPTIONS_DICT = {
    -1001: DisconnectedException,
    -1000: UnknownExchangeException,
    -1014: UnknownOrderCompositionException,
    -1015: TooManyOrdersException,
    -1003: TooManyRequestsException,
    -1007: TimeoutAPIException,
    -1002: MissingAPIKeyException,
    -1024: MissingSignatureException,
    -1023: MissingTimestampException,
    -1022: InvalidSignatureException,
    -1021: InvalidTimestampException,
    -1006: UnexpectedResponseException,
    -1020: UnsupportedOperationException,
    -1004: NoThisCompanyException,
    -1017: NoContentTypeException,
    -1016: ServiceShuttingDownException,
    -1121: InvalidContractException,
    -1117: InvalidSideException,
    -1116: InvalidSpotOrderTypeException,
    -1112: NoPendingOrdersException,
    -1138: OrderPriceRangeException,
    -1145: NotCancellableOrderException,
    -1106: NotRequiredParamException,
    -1103: UnknownParamException,
    -1102: RequiredParamException,
    -1105: RequiredParamEmptyException,
    -1104: UnreadParamException,
    -1147: OrderMaximumLimitException,
    -1136: OrderVolumeNotEnoughException,
    -1111: BadPrecisionException,
    -1100: IllegalCharsException,
    -1101: TooManyParametersException,
    -1139: MarketTradingDisabledException,
    35: RestrictedTransactionsException,
    -2016: FrozenTransactionsException,
    -2200: IllegalIPException,
    -2100: WrongParameterException,
    -2017: InsufficientBalanceException,
    -2013: OrderNotExistException,
    -2015: IncorrectAPIOrIPException,
}



from decimal import Decimal

from pydantic import BaseModel


class FuturesKlineData(BaseModel):
    """
    An object that contains information about the kline data.
    """

    idx: int
    """Open time"""
    open: Decimal
    """Open price"""
    close: Decimal
    """Close price"""
    high: Decimal
    """Highest price"""
    low: Decimal
    """Lowest price"""
    vol: Decimal
    """Trade volume"""



from enum import Enum


# TODO: check enum values
class SpotOrderStatus(str, Enum):
    NEW = "NEW"
    NEW_ORDER = "New Order"

    PARTIALLY_FILLED = "Partially Filled"
    FILLED = "Filled"
    CANCELLED = "Cancelled"
    TO_BE_CANCELLED = "PENDING_CANCEL"
    PARTIALLY_FILLED_OR_CANCELLED = "Partially Filled/Cancelled"
    REJECTED = "REJECTED"



from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.requests.base import BaseRequest
from odyssey_exchange_api.responses import SpotPairListResponse


class SpotPairListRequest(BaseRequest[SpotPairListResponse]):
    """
    The supported symbol pair collection which in the exchange. Returns a :class:`odyssey_exchange_api.responses.SpotPairListResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#symbol-pair-list
    """

    _request_url = BASE_SPOT_URL
    _request_method = "GET"
    _request_path = "/sapi/v1/symbols"

    __returning__ = SpotPairListResponse



from .asset_balance import SpotAssetBalance
from .asset_pair import SpotAssetPair
from .kline_data import SpotKlineData
from .my_trade import SpotMyTrade
from .order import SpotOrder
from .recent_trade import SpotRecentTrade
from .server_time import SpotServerTime
from .ticker_data import SpotTickerData
from .single_batch_order import SpotSingleBatchOrder



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.objects import FuturesOrder
from odyssey_exchange_api.requests.base import SignedRequest


class FuturesOpenOrdersRequest(SignedRequest[list[FuturesOrder]]):
    """
    Get a list of open orders. Returns an array of :class:`odyssey_exchange_api.objects.FuturesOrder`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#current-order-information
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "GET"
    _request_path = "/fapi/v1/openOrders"

    __returning__ = list[FuturesOrder]

    contract_name: str = Field(serialization_alias="contractName")
    """The lowercase symbol name, e.g., btcusdt."""



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.requests.base import SignedRequest, ResponseType
from odyssey_exchange_api.responses import FuturesCurrentTriggerOrdersResponse


class FuturesCurrentTriggerOrdersRequest(SignedRequest[FuturesCurrentTriggerOrdersResponse]):
    """
    Get all trigger orders. Returns a :class:`odyssey_exchange_api.responses.FuturesCurrentTriggerOrdersResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#obtain-the-current-trigger-order
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "POST"
    _request_path = "/fapi/v1/trigger_order_list"

    __returning__ = FuturesCurrentTriggerOrdersResponse

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    page: int
    """Current page number"""
    limit: int
    """Total data per page, maximum is 1000"""

    def make_response(self, data) -> ResponseType:
        return super().make_response(data["data"])



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.objects import FuturesTickerData
from odyssey_exchange_api.requests.base import BaseRequest


class FuturesTickerDataRequest(BaseRequest[FuturesTickerData]):
    """
    Retrieve 24-hour price change data, returns a :class:`odyssey_exchange_api.objects.FuturesTickerData`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#id-24hrs-ticker
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "GET"
    _request_path = "/fapi/v1/ticker"

    __returning__ = FuturesTickerData

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""



import re
from typing import Type

from odyssey_exchange_api.responses import WebsocketHistoryTradeResponse, WebsocketResponse, \
    WebsocketKlineResponse, WebsocketTickerResponse, WebsocketRealTimeTradeResponse
from odyssey_exchange_api.responses.websocket import WebsocketPingResponse, WebsocketFullDepthResponse


def resolve_websocket_response_class(data: dict) -> Type[WebsocketResponse] | None:
    stream_responses_obj = [
        WebsocketFullDepthResponse,
        WebsocketKlineResponse,
        WebsocketRealTimeTradeResponse,
        WebsocketTickerResponse,
    ]

    request_responses_obj = [
        WebsocketHistoryTradeResponse,
        WebsocketKlineResponse,
    ]

    if data.get("ping"):
        return WebsocketPingResponse

    elif data.get("event_rep"):
        current_responses_list = request_responses_obj
    else:
        current_responses_list = stream_responses_obj

    for response_obj in current_responses_list:
        pattern = re.compile(response_obj.__resolve_pattern__)
        if pattern.match(data.get("channel")):
            return response_obj



from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.requests.base import SignedRequest


from odyssey_exchange_api.responses import FuturesAccountInfoResponse


class FuturesAccountInfoRequest(SignedRequest[FuturesAccountInfoResponse]):
    """
    Get account information. Returns a :class:`odyssey_exchange_api.responses.FuturesAccountInfoResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#account-information
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "GET"
    _request_path = "/fapi/v1/account"

    __returning__ = FuturesAccountInfoResponse



from .ping import FuturesPingRequest
from .server_time import FuturesServerTimeRequest
from .contracts_list import FuturesContractsListRequest
from .depth import FuturesDepthRequest
from .ticker_data import FuturesTickerDataRequest
from .index_tag import FuturesIndexTagPriceRequest
from .kline_data import FuturesKlineDataRequest
from .create_order import FuturesCreateOrderRequest
from .create_condition_order import FuturesCreateConditionOrderRequest
from .cancel_order import FuturesCancelOrderRequest
from .order_information import FuturesOrderInfoRequest
from .open_orders import FuturesOpenOrdersRequest
from .historical_commission import FuturesHistoricalCommissionRequest
from .profit_and_loss import FuturesProfitAndLossRequest
from .my_trades import FuturesMyTradesRequest
from .change_position_model import FuturesChangePositionModelRequest
from .change_margin_model import FuturesChangeMarginModelRequest
from .change_leverage import FuturesChangeLeverageRequest
from .current_trigger_orders import FuturesCurrentTriggerOrdersRequest
from .cancel_trigger_order import FuturesCancelTriggerOrderRequest
from .account_info import FuturesAccountInfoRequest



from pydantic import BaseModel

from odyssey_exchange_api.objects import SpotAssetPair


class SpotPairListResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.SpotPairListRequest`.

    Contains a data of asset pairs at exchange.
    """

    symbols: list[SpotAssetPair]
    """Array of :class:`odyssey_exchange_api.objects.SpotAssetPair`"""



from decimal import Decimal

from pydantic import BaseModel, Field

from .account_positions import FuturesAccountPositions


class FuturesAccountCoinInfo(BaseModel):
    """
    An object that contains information about the coin at user's account.
    """

    margin_coin: str = Field(alias="marginCoin")
    """Margin currency"""
    account_normal: Decimal = Field(alias="accountNormal")
    """Account balance"""
    account_lock: Decimal = Field(alias="accountLock")
    """Margin frozen account"""
    part_position_normal: Decimal = Field(alias="partPositionNormal")
    """Isolated margin balance"""
    total_position_normal: Decimal = Field(alias="totalPositionNormal")
    """Initial margin occupied by full position"""
    achieved_amount: Decimal = Field(alias="achievedAmount")
    """Realized profit and loss"""
    unrealized_amount: Decimal = Field(alias="unrealizedAmount")
    """Unrealized profit or loss"""
    total_margin_rate: Decimal = Field(alias="totalMarginRate")
    """Cross Margin Rate"""
    total_equity: Decimal = Field(alias="totalEquity")
    """Cross Margin Interest"""
    part_equity: Decimal = Field(alias="partEquity")
    """Isolated interest"""
    total_cost: Decimal = Field(alias="totalCost")
    """Cost of occupying the entire warehouse"""
    sum_margin_rate: Decimal = Field(alias="sumMarginRate")
    """Margin rate for all accounts"""
    position_vos: list[FuturesAccountPositions] = Field(alias="positionVos")
    """Position contract records"""



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.requests.base import BaseRequest, SignedRequest
from odyssey_exchange_api.responses import FuturesOrderIDResponse


class FuturesCancelOrderRequest(SignedRequest[FuturesOrderIDResponse]):
    """
    Cancel an order. Returns an array of :class:`odyssey_exchange_api.responses.FuturesOrderIDResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#cancel-order
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "POST"
    _request_path = "/fapi/v1/cancel"

    __returning__ = FuturesOrderIDResponse

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    order_id: str = Field(serialization_alias="orderId")
    """Order ID."""



from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.objects import SpotTickerData
from odyssey_exchange_api.requests.base import BaseRequest


class SpotTickerDataRequest(BaseRequest[SpotTickerData]):
    """
    Retrieve 24-hour price change data, returns a :class:`odyssey_exchange_api.objects.SpotTickerData`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#id-24hrs-ticker
    """

    _request_url = BASE_SPOT_URL
    _request_method = "GET"
    _request_path = "/sapi/v1/ticker"

    __returning__ = SpotTickerData

    symbol: str
    """The uppercase symbol name, e.g., BTCUSDT."""



from pydantic import BaseModel, field_validator, Field

from odyssey_exchange_api.enums import SpotOrderStatus


class SpotCancelOrderResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.SpotCancelOrderRequest`.

    Contains an order id, asset symbol and order status.
    """

    order_id: str = Field(alias="orderId")
    """Order ID"""
    symbol: str
    """The lowercase symbol name, e.g., btcusdt."""
    status: SpotOrderStatus
    """Order status"""

    @field_validator("order_id", mode="before")
    @classmethod
    def retrieve_order_id(cls, v: str | list[str]) -> str:
        if isinstance(v, list):
            return v[0]
        return v



from decimal import Decimal

from pydantic import BaseModel

from odyssey_exchange_api.enums import SpotOrderSide


class SpotRecentTrade(BaseModel):
    """
    An object that contains information about recent trades.
    """

    side: SpotOrderSide
    """Order side"""
    price: Decimal
    """Trade price"""
    qty: Decimal
    """The quantity traded"""
    time: int
    """Trade timestamp"""



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.objects import FuturesMyTrade
from odyssey_exchange_api.requests.base import SignedRequest


class FuturesMyTradesRequest(SignedRequest[list[FuturesMyTrade]]):
    """
    Receives data on the transactions records. Returns an array of :class:`odyssey_exchange_api.objects.FuturesMyTrade`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#transaction-record
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "GET"
    _request_path = "/fapi/v1/myTrades"

    __returning__ = list[FuturesMyTrade]

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    limit: int = 100
    """The maximum number of items that can be returned in the query result. The value must be a positive integer. The default value is 100, maximum value is 1000."""
    from_id: int | None = Field(serialization_alias="fromId", default=None)
    """Start searching from this record"""



from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.objects import SpotOrder
from odyssey_exchange_api.requests.base import SignedRequest


class SpotOpenOrdersRequest(SignedRequest[list[SpotOrder]]):
    """
    Get a list of open orders. Returns an array of :class:`odyssey_exchange_api.objects.SpotOrder`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#current-open-orders
    """

    _request_url = BASE_SPOT_URL
    _request_method = "GET"
    _request_path = "/sapi/v1/openOrders"

    __returning__ = list[SpotOrder]

    symbol: str
    """The lowercase symbol name, e.g., btcusdt."""
    limit: int = 100
    """The maximum number of items that can be returned in the query result. The value must be a positive integer. The default value is 100, maximum value is 1000."""



from pydantic import BaseModel, Field


class SpotAssetPair(BaseModel):
    """
    An object that contains information about the asset.
    """

    symbol: str
    """The lowercase symbol name, e.g., btcusdt."""
    base_asset: str = Field(alias="baseAsset")
    """Underlying asset for the symbol"""
    quote_asset: str = Field(alias="quoteAsset")
    """Quote asset for the symbol"""
    price_precision: int = Field(alias="pricePrecision")
    """Price accuracy"""
    quantity_precision: int = Field(alias="quantityPrecision")
    """Quantity accuracy"""



import datetime
from decimal import Decimal

from pydantic import BaseModel, Field


class WebsocketKlineData(BaseModel):
    """
    An object that contains information about the kline data.
    """

    id: int
    """Open time"""
    open: Decimal
    """Open price"""
    close: Decimal
    """Close price"""
    high: Decimal
    """Highest price"""
    low: Decimal
    """Lowest price"""
    vol: Decimal
    """Trade volume"""
    amount: Decimal
    """Trade amount"""
    ds: datetime.datetime
    """Trade date"""
    trade_id: int = Field(validation_alias="tradeId")
    """Trade ID"""



from decimal import Decimal

from pydantic import BaseModel, Field

from odyssey_exchange_api.enums import FuturesContractSide, FuturesContractStatus, FuturesContractType


class FuturesContract(BaseModel):
    """
    An object that contains information about the futures contract.
    """

    symbol: str
    """The uppercase contract name, e.g., E-BTC-USDT."""
    contract_id: int = Field(alias="contractId")
    """Contract ID"""
    price_precision: int = Field(alias="pricePrecision")
    """Price accuracy"""
    status: FuturesContractStatus
    """Contract status"""
    type: FuturesContractType
    """Contract type"""
    side: FuturesContractSide
    """Contract direction"""
    multiplier: Decimal
    """Contract value"""
    multiplierCoin: str
    """Contract coin"""
    min_order_volume: Decimal = Field(alias="minOrderVolume")
    """Minimum order quantity"""
    min_order_money: Decimal = Field(alias="minOrderMoney")
    """Minimum order amount"""
    max_market_volume: Decimal = Field(alias="maxMarketVolume")
    """Maximum order quantity for market orders"""
    max_market_money: Decimal = Field(alias="maxMarketMoney")
    """Maximum order amount at market price"""
    max_limit_volume: Decimal = Field(alias="maxLimitVolume")
    """Maximum order quantity for limit orders"""
    max_limit_money: Decimal = Field(alias="maxLimitMoney")
    """Limit price maximum order amount"""
    max_valid_orders: Decimal = Field(alias="maxValidOrder")
    """The maximum number of valid orders"""



from .futures import FuturesContractStatus, FuturesContractType, FuturesContractSide, FuturesOrderStatus, \
    FuturesOrderType, FuturesOrderSide, FuturesPositionType, FuturesPositionSide, FuturesTimeInForce, \
    FuturesTriggerType, FuturesPositionModel, FuturesTriggerOrderStatus, FuturesPositionStatus, \
    FuturesPositionFrozenStatus
from .spot import SpotOrderStatus, SpotOrderSide, SpotOrderType
from .websocket import WebsocketOrderSide, WebsocketEventType



from .account_info import FuturesAccountInfoResponse
from .create_condition_order import FuturesCreateConditionOrderResponse
from .current_trigger_orders import FuturesCurrentTriggerOrdersResponse
from .depth import FuturesDepthResponse
from .order_id import FuturesOrderIDResponse



from .futures import FuturesDepthResponse, FuturesOrderIDResponse, FuturesCreateConditionOrderResponse, \
    FuturesCurrentTriggerOrdersResponse, FuturesAccountInfoResponse
from .spot import SpotAccountInfoResponse, SpotDepthResponse, SpotPairListResponse, SpotBatchOrdersResponse, \
    SpotCancelOrderResponse, SpotBatchCancelOrderResponse
from .websocket import WebsocketResponse, WebsocketHistoryTradeResponse, WebsocketPingResponse, \
    WebsocketFullDepthResponse, WebsocketKlineResponse, WebsocketRealTimeTradeResponse, WebsocketTickerResponse



import hashlib
import hmac
import urllib.parse
from typing import TypeVar, Any, Generic

from httpx import Request
from pydantic import BaseModel, TypeAdapter

ResponseType = TypeVar("ResponseType", bound=Any)
RequestDataType = TypeVar("RequestDataType", bound=Any)


class BaseRequest(BaseModel, Generic[ResponseType]):
    _request_url: str
    _request_method: str
    _request_path: str
    _request_headers = {}

    __returning__ = ResponseType

    def get_data(self):
        return self.model_dump(exclude_none=True, by_alias=True)

    def get_json(self):
        return self.model_dump_json(exclude_none=True, by_alias=True)

    def build_request(self) -> Request:
        data = None
        params = None

        if self._request_method == "GET":
            params = self.get_data()
        elif self._request_method == "POST":
            data = self.get_json()

        request = Request(
            method=self._request_method,
            url=self._request_url + self._request_path,
            headers=self._request_headers,
            data=data,
            params=params
        )
        return request

    def make_response(self, data) -> ResponseType:
        adapter = TypeAdapter(self.__returning__)
        response = adapter.validate_python(data)
        return response


class SignedRequest(BaseRequest[ResponseType]):
    def sign(
            self,
            timestamp: str,
            api_key: str,
            secret_key: str
    ):
        if self._request_method == "GET":
            data = self.get_data()
            data = urllib.parse.urlencode(data)
            data = "?" + data if data else ""
        else:
            data = self.get_json()

        _raw_sign = f"{timestamp}{self._request_method}{self._request_path}{data}"

        m = hmac.new(secret_key.encode("utf-8"), _raw_sign.encode('utf-8'), hashlib.sha256)
        sign = m.hexdigest()

        self._request_headers["X-CH-APIKEY"] = api_key
        self._request_headers["X-CH-SIGN"] = sign
        self._request_headers["X-CH-TS"] = timestamp
        self._request_headers["Content-Type"] = "application/json"


class WebsocketRequest(BaseModel):
    def get_data(self):
        return self.model_dump(exclude_none=True, by_alias=True)

    def get_json(self):
        return self.model_dump_json(exclude_none=True, by_alias=True)

    def build_request_data(self) -> dict:
        return self.get_data()



from .base import BaseExchangeException


class IllegalCharsException(BaseExchangeException):
    code = -1100
    msg = "请求中存在非法字符"
    reason = "Illegal characters found in a parameter."


class TooManyParametersException(BaseExchangeException):
    code = -1101
    msg = "发送的参数太多"
    reason = "Too many parameters sent for this endpoint."


class RequiredParamException(BaseExchangeException):
    code = -1102
    msg = "Forced parameter XXX not sent, empty or incorrect format"
    reason = "The parameter is null, the required parameter not input or incorrect format of input parameters"


class UnknownParamException(BaseExchangeException):
    code = -1103
    msg = "发送了未知参数"
    reason = "Each request requires at least one parameter"


class UnreadParamException(BaseExchangeException):
    code = -1104
    msg = "并非所有发送的参数都被读取"
    reason = "Not all sent parameters were read; read '%s' parameter(s) but was sent '%s'."


class RequiredParamEmptyException(BaseExchangeException):
    code = -1105
    msg = "The parameter XXX is empty"
    reason = "The required parameter is empty"


class NotRequiredParamException(BaseExchangeException):
    code = -1106
    msg = "不需要发送此参数"
    reason = "Parameter '%s' sent when not required."


class BadPrecisionException(BaseExchangeException):
    code = -1111
    msg = "精度超过此资产定义的最大值"
    reason = "Precision is over the maximum defined for this asset."


class NoPendingOrdersException(BaseExchangeException):
    code = -1112
    msg = "There are no pending orders for trading pairs"
    reason = "The order that needs to be canceled does not exist."


class InvalidSpotOrderTypeException(BaseExchangeException):
    code = -1116
    msg = "Invalid order type"
    reason = "Invalid orderType. In the current version , ORDER_TYPE values is LIMIT or MARKET."


class InvalidSideException(BaseExchangeException):
    code = -1117
    msg = "Invalid buying or selling direction"
    reason = "Invalid side. ORDER_SIDE values is BUY or SELL"


class InvalidContractException(BaseExchangeException):
    code = -1121
    msg = "Invalid contract"
    reason = "Invalid symbol."


class OrderVolumeNotEnoughException(BaseExchangeException):
    code = -1136
    msg = "订单数量小于最小值"
    reason = "Order volume lower than the minimum."


class OrderPriceRangeException(BaseExchangeException):
    code = -1138
    msg = "The order price is outside the allowable range"
    reason = "Order price exceeds permissible range."


class MarketTradingDisabledException(BaseExchangeException):
    code = -1139
    msg = "The pair does not support market trading"
    reason = "This trading pair does not support market trading"


class NotCancellableOrderException(BaseExchangeException):
    code = -1145
    msg = "The order status does not allow for cancellation"
    reason = "The order cannot be canceled"


class OrderMaximumLimitException(BaseExchangeException):
    code = -1147
    msg = "价格或数量精度超过最大限制"
    reason = "Order price or quantity exceeds the maximum limit"



from decimal import Decimal

from pydantic import BaseModel, Field

from odyssey_exchange_api.enums import FuturesOrderStatus, FuturesOrderType, FuturesOrderSide


class FuturesOrder(BaseModel):
    """
    An object that contains information about the order.
    """

    order_id: int = Field(alias="orderId")
    """Order ID"""
    contract_name: str = Field(alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    transact_time: int | None = Field(default=None, alias="transactTime")
    """The time of place order"""
    price: Decimal
    """Order price"""
    original_quantity: Decimal = Field(alias="origQty")
    """Order volume"""
    executed_quantity: Decimal = Field(alias="executedQty")
    """The number of already deal order"""
    average_price: Decimal | None = Field(alias="avgPrice")
    """The average price of already deal order"""
    type: FuturesOrderType
    """Type of the order"""
    side: FuturesOrderSide
    """Side of the order"""
    status: FuturesOrderStatus
    """Order status"""
    time_in_force: str | int = Field(alias="timeInForce")
    """Effective method of conditional order"""



from .base import WebsocketResponse
from ...objects import WebsocketFullDepth


class WebsocketFullDepthResponse(WebsocketResponse):
    __resolve_pattern__ = r"^market_\w+_depth_step\d+$"

    channel: str
    ts: int
    tick: WebsocketFullDepth
    status: str



from decimal import Decimal

from pydantic import BaseModel


class WebsocketTickerData(BaseModel):
    """
    An object with information about an asset per day.
    """

    high: Decimal
    """Highest price"""
    low: Decimal
    """Lowest price"""
    vol: Decimal
    """Trade volume"""
    amount: Decimal
    """Trade amount"""
    rose: str
    """Range of increase and decrease, + is increase, - is decrease, +0.05 means increase by 5%"""



from pydantic import BaseModel, Field


class FuturesServerTime(BaseModel):
    """
    An object that contains information about the time zone and the current timestamp.
    """

    timezone: str
    """Server timezone"""
    server_time: int = Field(alias="serverTime")
    """Current timestamp"""



from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.objects import SpotSingleBatchOrder
from odyssey_exchange_api.requests.base import SignedRequest
from odyssey_exchange_api.responses import SpotBatchOrdersResponse


class SpotBatchOrdersRequest(SignedRequest[SpotBatchOrdersResponse]):
    """
    Place batch orders. Returns a :class:`odyssey_exchange_api.responses.SpotBatchOrdersResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#batch-orders
    """

    _request_url = BASE_SPOT_URL
    _request_method = "POST"
    _request_path = "/sapi/v1/batchOrders"

    __returning__ = SpotBatchOrdersResponse

    symbol: str
    """The uppercase symbol name, e.g., BTCUSDT."""
    orders: list[SpotSingleBatchOrder]
    """An array of orders, with a maximum of 10 orders in the array."""



from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.objects import SpotServerTime
from odyssey_exchange_api.requests.base import BaseRequest


class SpotServerTimeRequest(BaseRequest[SpotServerTime]):
    """
    This interface checks connectivity to the server and retrieves server timestamp. Returns a :class:`odyssey_exchange_api.objects.ServerTime`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#check-server-time
    """

    _request_url = BASE_SPOT_URL
    _request_method = "GET"
    _request_path = "/sapi/v1/time"

    __returning__ = SpotServerTime



from decimal import Decimal

from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.enums import FuturesOrderSide, FuturesOrderType, FuturesPositionSide, FuturesPositionType, \
    FuturesTriggerType
from odyssey_exchange_api.requests.base import SignedRequest, ResponseType
from odyssey_exchange_api.responses import FuturesCreateConditionOrderResponse


class FuturesCreateConditionOrderRequest(SignedRequest[FuturesCreateConditionOrderResponse]):
    """
    Create a new condition order. Returns a :class:`odyssey_exchange_api.responses.FuturesCreateConditionOrderResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#create-condition-order
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "POST"
    _request_path = "/fapi/v1/conditionOrder"

    __returning__ = FuturesCreateConditionOrderResponse

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    trigger_type: FuturesTriggerType = Field(serialization_alias="triggerType")
    """Type of condition."""
    trigger_price: Decimal = Field(serialization_alias="triggerPrice")
    """Trigger price."""
    volume: Decimal
    """The number of orders placed has a precision limit. When opening a position at market price, the unit here is value."""
    side: FuturesOrderSide
    """Order side."""
    type: FuturesOrderType
    """Order type."""
    open: FuturesPositionSide
    """Position side."""
    position_type: FuturesPositionType = Field(serialization_alias="positionType")
    """Position type."""

    price: Decimal | None = None
    """Order price, for LIMIT orders must be sent, there is a precision limit."""
    clientOrderId: str | None = None
    """Client order ID."""

    def make_response(self, data) -> ResponseType:
        return super().make_response(data["data"])



from pydantic import BaseModel

from odyssey_exchange_api.objects import FuturesAccountCoinInfo


class FuturesAccountInfoResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.FuturesAccountInfoRequest`.

    An object with information about requested account.
    """

    account: list[FuturesAccountCoinInfo]
    """Array with information about coins at account"""



from .base import WebsocketResponse

from .ping import WebsocketPingResponse
from .request_history_trade import WebsocketHistoryTradeResponse
from .full_depth import WebsocketFullDepthResponse
from .real_time_trade import WebsocketRealTimeTradeResponse
from .kline_history import WebsocketKlineResponse
from .ticker import WebsocketTickerResponse




from .account_info import SpotAccountInfoRequest
from .batch_cancel_order import SpotBatchCancelOrderRequest
from .batch_orders import SpotBatchOrdersRequest
from .cancel_order import SpotCancelOrderRequest
from .create_order import SpotCreateOrderRequest
from .create_order import SpotCreateOrderRequest
from .depth import SpotDepthRequest
from .kline_data import SpotKlineDataRequest
from .my_trades import SpotMyTradesRequest
from .open_orders import SpotOpenOrdersRequest
from .pair_list import SpotPairListRequest
from .ping import SpotPingRequest
from .query_order import SpotQueryOrderRequest
from .recent_trades import SpotRecentTradesRequest
from .server_time import SpotServerTimeRequest
from .test_create_order import SpotTestCreateOrderRequest
from .ticker_data import SpotTickerDataRequest



from decimal import Decimal

from pydantic import BaseModel, Field

from odyssey_exchange_api.enums import FuturesOrderSide, FuturesPositionType


class FuturesProfitAndLoss(BaseModel):
    """
    An object that contains information about profit and losses.
    """

    side: FuturesOrderSide
    """Order side"""
    position_type: FuturesPositionType = Field(validation_alias="positionType")
    """Position type"""
    trade_fee: Decimal = Field(validation_alias="tradeFee")
    """Trade fee"""
    realized_amount: Decimal = Field(validation_alias="realizedAmount")
    """Realized amount"""
    leverage_level: int = Field(validation_alias="leverageLevel")
    """Leverage level"""
    open_price: Decimal = Field(validation_alias="openPrice")
    """Open price"""
    settle_profit: Decimal | None = Field(validation_alias="settleProfit", default=None)
    mtime: int
    """Modify time"""
    share_amount: Decimal = Field(validation_alias="shareAmount")
    """Share amount"""
    open_end_price: Decimal = Field(validation_alias="openEndPrice")
    """Open end price"""
    close_profit: Decimal = Field(validation_alias="closeProfit")
    """Close profit"""
    volume: Decimal
    """Volume"""
    contract_id: int = Field(validation_alias="contractId")
    """Contract ID"""
    history_realized_amount: Decimal = Field(validation_alias="historyRealizedAmount")
    """History realized amount"""
    ctime: int
    """Creation time"""
    id: int
    """ID"""
    capital_fee: Decimal = Field(validation_alias="capitalFee")
    """Capital fee"""



from pydantic import Field

from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.requests.base import SignedRequest
from odyssey_exchange_api.responses import SpotCancelOrderResponse


class SpotCancelOrderRequest(SignedRequest[SpotCancelOrderResponse]):
    """
    Cancel an order. Returns a :class:`odyssey_exchange_api.responses.SpotCancelOrderResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#cancel-order
    """

    _request_url = BASE_SPOT_URL
    _request_method = "POST"
    _request_path = "/sapi/v1/cancel"

    __returning__ = SpotCancelOrderResponse

    order_id: str = Field(serialization_alias="orderId")
    """Order ID."""
    symbol: str
    """The lowercase symbol name, e.g., btcusdt."""



from pydantic import BaseModel, Field

from odyssey_exchange_api.objects import FuturesTriggerOrder


class FuturesCurrentTriggerOrdersResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.FuturesCurrentTriggerOrdersRequest`.

    An object with count and list of trigger orders.
    """

    count: int
    """Count of available trigger orders"""
    trigger_order_list: list[FuturesTriggerOrder] = Field(alias="trigOrderList")
    """Array with trigger order objects"""



from .pair_list import SpotPairListResponse
from .depth import SpotDepthResponse
from .batch_orders import SpotBatchOrdersResponse
from .cancel_order import SpotCancelOrderResponse
from .batch_cancel_order import SpotBatchCancelOrderResponse
from .account_info import SpotAccountInfoResponse




from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.objects import FuturesContract
from odyssey_exchange_api.requests.base import BaseRequest


class FuturesContractsListRequest(BaseRequest[list[FuturesContract]]):
    """
    Get a list of all contracts on the exchange. Returns an array of :class:`odyssey_exchange_api.objects.FuturesContract`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#contracts-list
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "GET"
    _request_path = "/fapi/v1/contracts"

    __returning__ = list[FuturesContract]



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.requests.base import SignedRequest, ResponseType


class FuturesChangeLeverageRequest(SignedRequest[bool]):
    """
    Change leverage. Returns a :class:`odyssey_exchange_api.responses.FuturesAccountInfoResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#account-information
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "POST"
    _request_path = "/fapi/v1/edit_lever"

    __returning__ = bool

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    now_level: int = Field(serialization_alias="nowLevel")
    """Leverage multiple to be modified."""

    def make_response(self, data) -> ResponseType:
        if data.get("code") == "0" and data.get("msg") == "成功":
            return True
        return False



from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.requests.base import BaseRequest


class SpotPingRequest(BaseRequest[dict]):
    """
    This interface checks connectivity to the host. Returns an empty :class:`dict`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#test-connectivity
    """

    _request_url = BASE_SPOT_URL
    _request_method = "GET"
    _request_path = "/sapi/v1/ping"

    __returning__ = dict



from .base import WebsocketResponse
from ...objects import WebsocketHistoryTrade


class WebsocketHistoryTradeResponse(WebsocketResponse):
    __resolve_pattern__ = r"^market_\w+_trade_ticker$"

    event_rep: str
    channel: str
    ts: int
    status: str
    data: list[WebsocketHistoryTrade]



from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.requests.base import SignedRequest
from odyssey_exchange_api.responses import SpotAccountInfoResponse


class SpotAccountInfoRequest(SignedRequest[SpotAccountInfoResponse]):
    """
    Get account information. Returns a :class:`odyssey_exchange_api.responses.SpotAccountInfoResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#account-information
    """

    _request_url = BASE_SPOT_URL
    _request_method = "GET"
    _request_path = "/sapi/v1/account"

    __returning__ = SpotAccountInfoResponse



from .contract_side import FuturesContractSide
from .contract_status import FuturesContractStatus
from .contract_type import FuturesContractType
from .order_side import FuturesOrderSide
from .order_status import FuturesOrderStatus
from .order_type import FuturesOrderType
from .position_frozen_status import FuturesPositionFrozenStatus
from .position_model import FuturesPositionModel
from .position_side import FuturesPositionSide
from .position_status import FuturesPositionStatus
from .position_type import FuturesPositionType
from .time_in_force import FuturesTimeInForce
from .trigger_order_status import FuturesTriggerOrderStatus
from .trigger_type import FuturesTriggerType



from .base import BaseExchangeException


class UnknownExchangeException(BaseExchangeException):
    code = -1000
    msg = "An unknown error occurred while processing the request"
    reason = "The wrong input parameter symbol name or input wrong contract name."


class DisconnectedException(BaseExchangeException):
    code = -1001
    msg = "内部错误; 无法处理您的请求。 请再试一次"
    reason = "Internal error; unable to process your request. Please try again."


class MissingAPIKeyException(BaseExchangeException):
    code = -1002
    msg = ("You are not authorized to execute this request. "
           "The request needs to send an API Key, and we recommend appending the X-CH-APIKEY to all request headers")
    reason = "The X_CH_APIKEY required for signature is not added to the header."


class TooManyRequestsException(BaseExchangeException):
    code = -1003
    msg = "请求过于频繁超过限制"
    reason = "Requests exceed the limit too frequently."


class NoThisCompanyException(BaseExchangeException):
    code = -1004
    msg = "您无权执行此请求 user not exit"
    reason = "You are not authorized to execute this request. User not exit Company."


class UnexpectedResponseException(BaseExchangeException):
    code = -1006
    msg = "接收到了不符合预设格式的消息，下单状态未知"
    reason = ("An unexpected response was received from the message bus. Execution status unknown. "
              "OPEN API server find some exception in execute request. Please report to Customer service.")


class TimeoutAPIException(BaseExchangeException):
    code = -1007
    msg = "等待后端服务器响应超时。 发送状态未知； 执行状态未知"
    reason = "Timeout waiting for response from backend server. Send status unknown; execution status unknown."


class UnknownOrderCompositionException(BaseExchangeException):
    code = -1014
    msg = "不支持的订单组合"
    reason = "Unsupported order combination."


class TooManyOrdersException(BaseExchangeException):
    code = -1015
    msg = "Too many orders. Please reduce the number of your orders"
    reason = "The order quantity exceeds the maximum quantity limit."


class ServiceShuttingDownException(BaseExchangeException):
    code = -1016
    msg = "服务器下线"
    reason = "This service is no longer available."


class NoContentTypeException(BaseExchangeException):
    code = -1017
    msg = "我们建议在所有的请求头附加Content-Type, 并设置成application/json"
    reason = "We recommend attaching Content-Type to all request headers and setting it to application/json."


class UnsupportedOperationException(BaseExchangeException):
    code = -1020
    msg = "不支持此操作"
    reason = "This operation is not supported."


class InvalidTimestampException(BaseExchangeException):
    code = -1021
    msg = "Invalid timestamp with too large time offset"
    reason = ("The timestamp offset is too large. Timestamp for this request was 1000ms ahead of the server's time. "
              "Please check the difference between your local time and server time.")


class InvalidSignatureException(BaseExchangeException):
    code = -1022
    msg = "Invalid signature"
    reason = "Signature verification failed"


class MissingTimestampException(BaseExchangeException):
    code = -1023
    msg = ("You are not authorized to execute this request. "
           "The request need to send timestamps, and we recommend appending X-CH-TS to all request headers")
    reason = "The X-CH-TS required for signature is not added to the header."


class MissingSignatureException(BaseExchangeException):
    code = -1024
    msg = ("You are not authorized to execute this request. "
           "The request needs to send sign, and we recommend appending X-CH-SIGN to all request headers")
    reason = "The X-CH-SIGN required for signature is not added to the header."



import datetime
from decimal import Decimal

from pydantic import BaseModel

from odyssey_exchange_api.enums import WebsocketOrderSide


class WebsocketHistoryTrade(BaseModel):
    """
    An object that contains information about history trades.
    """

    side: WebsocketOrderSide
    """Order side"""
    price: Decimal
    """Trade price"""
    amount: Decimal
    """The volume of the base coin"""
    vol: Decimal
    """the volume of the main coin"""
    ts: int
    """Trade timestamp"""
    ds: datetime.datetime
    """Trade date"""



from pydantic import Field

from odyssey_exchange_api.base import BASE_SPOT_URL
from odyssey_exchange_api.requests.base import SignedRequest
from odyssey_exchange_api.responses import SpotBatchCancelOrderResponse


class SpotBatchCancelOrderRequest(SignedRequest[SpotBatchCancelOrderResponse]):
    """
    Mass cancellation of orders, maximum 10 orders at a time. Returns a :class:`odyssey_exchange_api.responses.SpotBatchCancelOrderResponse`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/spot-trading-api#batch-cancel-order
    """

    _request_url = BASE_SPOT_URL
    _request_method = "POST"
    _request_path = "/sapi/v1/batchCancel"

    __returning__ = SpotBatchCancelOrderResponse

    order_ids: list[int] = Field(serialization_alias="orderIds")
    """Array of order ids."""
    symbol: str
    """The uppercase symbol name, e.g., BTCUSDT."""



from pydantic import BaseModel

from .base import WebsocketResponse
from ...objects import WebsocketHistoryTrade


class WebsocketRealTimeTradeTickData(BaseModel):
    data: list[WebsocketHistoryTrade]
    ts: int


class WebsocketRealTimeTradeResponse(WebsocketResponse):
    __resolve_pattern__ = r"^market_\w+_trade_ticker$"

    channel: str
    ts: int
    tick: WebsocketRealTimeTradeTickData
    data: None
    status: str



from .futures import FuturesPingRequest, FuturesDepthRequest, FuturesContractsListRequest, FuturesServerTimeRequest, \
    FuturesTickerDataRequest, FuturesIndexTagPriceRequest, FuturesKlineDataRequest, FuturesCreateOrderRequest, \
    FuturesCreateConditionOrderRequest, FuturesCancelOrderRequest, FuturesOrderInfoRequest, FuturesOpenOrdersRequest, \
    FuturesHistoricalCommissionRequest, FuturesProfitAndLossRequest, FuturesMyTradesRequest, \
    FuturesChangePositionModelRequest, FuturesChangeMarginModelRequest, FuturesChangeLeverageRequest, \
    FuturesCurrentTriggerOrdersRequest, FuturesCancelTriggerOrderRequest, FuturesAccountInfoRequest
from .spot import SpotDepthRequest, SpotMyTradesRequest, SpotPingRequest, SpotOpenOrdersRequest, \
    SpotCancelOrderRequest, SpotBatchCancelOrderRequest, SpotCreateOrderRequest, SpotBatchOrdersRequest, \
    SpotTestCreateOrderRequest, SpotRecentTradesRequest, SpotPairListRequest, SpotAccountInfoRequest, \
    SpotQueryOrderRequest, SpotKlineDataRequest, SpotTickerDataRequest, SpotServerTimeRequest
from .websockets import WebsocketPongRequest, WebsocketHistoryTradeRequest, WebsocketKlineHistoryRequest, \
    WebsocketKlineMarketRequest, WebsocketMarketTickerRequest, WebsocketRealTimeTradeRequest, \
    WebsocketFullDepthRequest



from odyssey_exchange_api.requests.base import WebsocketRequest


class WebsocketPongRequest(WebsocketRequest):
    """
    PONG is the response to the PING request.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/websocket#heartbeat
    """
    pong: int



from decimal import Decimal

from pydantic import BaseModel, Field, field_validator

from odyssey_exchange_api.enums import SpotOrderStatus, SpotOrderType, SpotOrderSide


class SpotOrder(BaseModel):
    """
    An object that contains information about the order.
    """

    order_id: str = Field(alias="orderId")
    """Order ID"""
    client_order_id: str | None = Field(default=None, alias="clientOrderId")
    """User order ID"""
    symbol: str
    """The uppercase symbol name, e.g., BTCUSDT."""
    transact_time: int | None = Field(default=None, alias="transactTime")
    """The time of place order"""
    price: Decimal
    """Order price"""
    original_quantity: Decimal = Field(alias="origQty")
    """Order volume"""
    executed_quantity: Decimal = Field(alias="executedQty")
    """The number of already deal order"""
    average_price: Decimal | None = Field(alias="avgPrice", default=None)
    """The average price of already deal order"""
    type: SpotOrderType
    """Type of the order"""
    side: SpotOrderSide
    """Side of the order"""
    status: SpotOrderStatus
    """Order status"""

    @field_validator("order_id", mode="before")
    @classmethod
    def retrieve_order_id(cls, v: str | list[str] | int) -> str:
        if isinstance(v, list):
            return v[0]
        return str(v)



import datetime
from decimal import Decimal

from pydantic import BaseModel, Field

from odyssey_exchange_api.enums import FuturesOrderSide, FuturesPositionType, FuturesPositionSide


class FuturesHistoricalCommission(BaseModel):
    """
    An object that contains information about the historical commission at positions.
    """

    side: FuturesOrderSide
    """Order side"""
    client_id: str = Field(validation_alias="clientId")
    ctime_ms: int = Field(validation_alias="ctimeMs")
    """Creation time at milliseconds"""
    position_type: FuturesPositionType = Field(validation_alias="positionType")
    """Position type"""
    order_id: int = Field(validation_alias="orderId")
    """Order ID"""
    average_price: Decimal | None = Field(validation_alias="avgPrice", default=None)
    """Average order price"""
    open_or_close: FuturesPositionSide = Field(validation_alias="openOrClose")
    """Position side"""
    leverage_level: int = Field(validation_alias="leverageLevel")
    """Leverage level"""
    type: int
    close_taker_fee_rate: Decimal = Field(validation_alias="closeTakerFeeRate")
    """Close taker fee rate"""
    open_maker_fee_rate: Decimal = Field(validation_alias="openMakerFeeRate")
    """Open maker fee rate"""
    close_maker_fee_rate: Decimal = Field(validation_alias="closeMakerFeeRate")
    """Close maker fee rate"""
    open_taker_fee_rate: Decimal = Field(validation_alias="openTakerFeeRate")
    """Open taker fee rate"""
    volume: Decimal
    """Volume"""
    deal_volume: Decimal = Field(validation_alias="dealVolume")
    """Deal volume"""
    price: Decimal | None = None
    """Price"""
    contract_id: int = Field(validation_alias="contractId")
    """Contract ID"""
    ctime: datetime.datetime
    """Creation date"""
    contractName: str = Field(validation_alias="contractName")
    """Contract name"""
    deal_money: Decimal = Field(validation_alias="dealMoney")
    """Deal money"""
    status: int
    """Status"""



from decimal import Decimal

from pydantic import BaseModel, Field

from odyssey_exchange_api.enums import FuturesOrderSide, FuturesPositionSide, FuturesTriggerType, \
    FuturesTriggerOrderStatus


class FuturesTriggerOrder(BaseModel):
    """
    An object that contains information about the trigger order.
    """

    id: int
    """Trigger order collection table id"""
    trigger_order_id: int = Field(alias="triggerOrderId")
    """Trigger order ID"""
    contract_id: int = Field(alias="contractId")
    """Contract ID"""
    contract_name: str = Field(alias="contractName")
    """Contract name"""
    margin_coin: str = Field(alias="marginCoin")
    """Margin currenct"""
    multiplier: Decimal
    """Contract value"""
    trigger_price: Decimal = Field(alias="triggerPrice")
    """Trigger price"""
    price: Decimal
    """Price"""
    price_precision: int = Field(alias="pricePrecision")
    """Price accuracy"""
    volume: Decimal
    """Order volume"""
    open: FuturesPositionSide
    """Direction of opening and closing positions"""
    side: FuturesOrderSide
    """Order side"""
    status: FuturesTriggerOrderStatus
    """Valid status"""
    expire_time: int = Field(alias="expireTime")
    """Order expiration time"""
    ctime: int
    """Creation time"""
    mtime: int
    """Modify time"""
    trigger_type: FuturesTriggerType = Field(alias="triggerType")
    """Trigger order type"""



from decimal import Decimal

from pydantic import BaseModel, Field

from odyssey_exchange_api.enums import SpotOrderSide


class SpotMyTrade(BaseModel):
    """
    An object that contains information about my trades.
    """

    symbol: str
    """The uppercase symbol name, e.g., BTCUSDT."""
    id: int
    """Trade ID"""
    bid_id: int = Field(alias="bidId")
    """Buy side order ID"""
    ask_id: int = Field(alias="askId")
    """Sell side order ID"""
    price: Decimal
    """Trade price"""
    quantity: Decimal = Field(alias="qty")
    """Trade volume"""
    time: int
    """Trade timestamp"""
    is_buyer: bool = Field(alias="isBuyer")
    """if True it is buyer, else seller"""
    is_maker: bool = Field(alias="isMaker")
    """If True it is Maker, else taker"""
    fee_coin: str = Field(alias="feeCoin")
    """Trade fee coin"""
    fee: Decimal
    """Trade fee"""
    bid_user_id: int = Field(alias="bidUserId")
    """Buy side user id"""
    ask_user_id: int = Field(alias="askUserId")
    """Sell side user id"""
    is_self: bool = Field(alias="isSelf")
    """Self deal status, true if self deal, else not self deal"""
    side: SpotOrderSide
    """Order side"""



from pydantic import BaseModel, Field

from .account_position import FuturesAccountPosition


class FuturesAccountPositions(BaseModel):
    """
    An object that contains information about contracts and contract positions.
    """

    contract_id: int = Field(alias="contractId")
    """Contract id"""
    contract_name: str = Field(alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    contract_other_name: str = Field(alias="contractOtherName")
    """Currency alias"""
    contract_symbol: str = Field(alias="contractSymbol")
    """Contract currency pair"""
    positions: list[FuturesAccountPosition]
    """Position detail information"""



import setuptools

setuptools.setup(
    name="odyssey_exchange_api",
    version="0.0.3",
    author="goduni",
    url="https://github.com/Odyssey-Trade/python-odyssey-exchange-api",
    description="Library for working with odyssey.trade",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages("odyssey_exchange_api"),
    package_dir={"": "odyssey_exchange_api"},
    python_requires=">=3.10",
    install_requires=open("requirements.txt").read().splitlines(),
    license="MIT"
)



from .base import WebsocketResponse
from ...objects import WebsocketKlineData


class WebsocketKlineResponse(WebsocketResponse):
    __resolve_pattern__ = r'^market_\w+_kline_\w+$'

    channel: str
    ts: int
    data: list[WebsocketKlineData] | None
    tick: WebsocketKlineData | None
    status: str



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.requests.base import SignedRequest, ResponseType


class FuturesCancelTriggerOrderRequest(SignedRequest[bool]):
    """
    Cancel trigger order. Returns a :class:`bool`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#cancel-trigger-order
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "POST"
    _request_path = "/v1/inner/trigger_order_cancel"

    __returning__ = bool

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    order_id: str = Field(serialization_alias="orderId")
    """Order ID."""

    def make_response(self, data) -> ResponseType:
        if data.get("code") == "0" and data.get("msg") == "成功":
            return True
        return False



from pydantic import BaseModel


class SpotBatchCancelOrderResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.SpotBatchCancelOrderRequest`.

    Contains an arrays with successful cancelled order ids and failed order ids.

    Cancellation failure is usually because the order does not exist or the order status has reached the final state.
    """

    success: list[int] = []
    """An array of ID orders that have been canceled"""
    failed: list[int] = []
    """An array of ID orders that could not be canceled"""



from .full_depth import WebsocketFullDepthRequest
from .history_trade import WebsocketHistoryTradeRequest
from .kline_history import WebsocketKlineHistoryRequest
from .kline_market import WebsocketKlineMarketRequest
from .market_ticker import WebsocketMarketTickerRequest
from .ping import WebsocketPongRequest
from .real_time_trade import WebsocketRealTimeTradeRequest



from enum import Enum


class FuturesOrderStatus(str, Enum):
    INIT = "INIT"

    NEW = "NEW"
    NEW_ORDER = "New Order"

    PARTIALLY_FILLED = "PART_FILLED"
    FILLED = "FILLED"
    CANCELLED = "Cancelled"
    TO_BE_CANCELLED = "PENDING_CANCEL"
    PARTIALLY_FILLED_OR_CANCELLED = "Partially Filled/Cancelled"
    REJECTED = "REJECTED"



from typing import Any


class UnknownException(Exception):
    pass


class BaseExchangeException(Exception):
    code: int
    msg: str
    reason: str
    data: Any

    def __init__(self, data: Any = None):
        super().__init__(self.msg)
        self.data = data



from .base import BaseExchangeException


class OrderNotExistException(BaseExchangeException):
    code = -2013
    msg = "订单不存在"
    reason = "Order does not exist."


class IncorrectAPIOrIPException(BaseExchangeException):
    code = -2015
    msg = "Invalid API key, IP or operation permission"
    reason = "Signature or IP is incorrect"


class FrozenTransactionsException(BaseExchangeException):
    code = -2016
    msg = "Transactions are frozen"
    reason = "The user's transaction is frozen"


class InsufficientBalanceException(BaseExchangeException):
    code = -2017
    msg = "余额不足"
    reason = "Insufficient balance"


class WrongParameterException(BaseExchangeException):
    code = -2100
    msg = "Parameter error"
    reason = "Input wrong parameter or exist block in the parameter"


class IllegalIPException(BaseExchangeException):
    code = -2200
    msg = "Illegal IP"
    reason = "Not trusted IP"


class RestrictedTransactionsException(BaseExchangeException):
    code = 35
    msg = "禁止下单(Forbidden to order)"
    reason = "User transactions may be restricted"



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.objects import FuturesOrder
from odyssey_exchange_api.requests.base import SignedRequest


class FuturesOrderInfoRequest(SignedRequest[FuturesOrder]):
    """
    Get the order data. Returns a :class:`odyssey_exchange_api.objects.FuturesOrder`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#order-information
    """
    _request_url = BASE_FUTURES_URL
    _request_method = "GET"
    _request_path = "/fapi/v1/order"

    __returning__ = FuturesOrder

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    order_id: str = Field(serialization_alias="orderId")
    """Order ID."""
    client_order_id: str | None = Field(serialization_alias="clientOrderId", default=None)
    """Client order ID."""



from pydantic import BaseModel, Field


class FuturesCreateConditionOrderResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.FuturesCreateConditionOrderRequest`.

    Contains an arrays with trigger_ids, ids, cancel_ids.
    """

    ids: list[int]
    """Array with order ids"""
    trigger_ids: list[int] = Field(validation_alias="triggerIds")
    """Array with trigger order ids"""
    cancel_ids: list[int] = Field(validation_alias="cancelIds")
    """Array with cancelled order ids"""



from .base import WebsocketResponse
from ...objects import WebsocketTickerData


class WebsocketTickerResponse(WebsocketResponse):
    __resolve_pattern__ = r'^market_\w+_ticker$'

    event_rep: str
    channel: str
    data: None
    tick: WebsocketTickerData
    status: str



from decimal import Decimal

from pydantic import BaseModel, Field


class FuturesIndexTagPrice(BaseModel):
    """
    An object that contains information about the index price, tag price and funding data.
    """

    index_price: Decimal = Field(alias="indexPrice")
    """Index price"""
    tag_price: Decimal = Field(alias="tagPrice")
    """Tag price"""
    next_fund_rate: Decimal = Field(alias="nextFundRate")
    """Funding rate price"""
    current_fund_rate: Decimal = Field(alias="currentFundRate")
    """Funding rate price for the previous period (used for settlement in this period)"""



from pydantic import Field

from odyssey_exchange_api.base import BASE_FUTURES_URL
from odyssey_exchange_api.objects import FuturesKlineData
from odyssey_exchange_api.requests.base import BaseRequest


class FuturesKlineDataRequest(BaseRequest[list[FuturesKlineData]]):
    """
    Retrieve Kline/candlestick data. Returns an array of :class:`FuturesKlineData`.

    Source: https://exchangeopenapi.gitbook.io/pri-openapi/openapi-doc/futures-trading-api#kline-candlestick-data
    """

    _request_url = BASE_FUTURES_URL
    _request_method = "GET"
    _request_path = "/fapi/v1/klines"

    __returning__ = list[FuturesKlineData]

    contract_name: str = Field(serialization_alias="contractName")
    """The uppercase contract name, e.g., E-BTC-USDT."""
    interval: str
    """Interval of the Kline. Possible values include: 1min, 5min, 15min, 30min, 60min, 1day, 1week, 1month."""
    limit: int = 100
    """The maximum number of items that can be returned in the query result. The value must be a positive integer. The default value is 100, maximum value is 300."""



from pydantic import BaseModel

from odyssey_exchange_api.objects import SpotAssetBalance


class SpotAccountInfoResponse(BaseModel):
    """
    Response of the request: :class:`odyssey_exchange_api.requests.SpotAccountInfoRequest`.

    Contains an account balances.
    """

    balances: list[SpotAssetBalance]
    """Array of :class:`odyssey_exchange_api.objects.SpotAssetBalance`."""



from decimal import Decimal

from pydantic import BaseModel, Field

from odyssey_exchange_api.enums import SpotOrderSide, SpotOrderType


class SpotSingleBatchOrder(BaseModel):
    """
    An object with information about a single order in batch orders.
    """

    volume: Decimal
    """Order volume"""
    side: SpotOrderSide
    """Order side"""
    batch_type: SpotOrderType = Field(serialization_alias="batchType")
    """Order type"""

    price: Decimal | None = None
    """Order price"""
